// Menu.c
// This is a generic Menu handler
// see Menu.h for description of usage


#include "Project.h"
#include "Menu_IO.h"

#ifdef EN_YES_NO
	#define EN_SELECT_1_ON_LINE
#endif

// function in main to execute the selected operation
void Execute (Menu_t * pointer, uint8_t);

// local functions
void FillIndexList ( Menu_t * pointer, uint8_t );
void ShowMenuEntries ( Menu_t * pointer, uint8_t );

// menu variables
uint8_t MenuLevel;
char	MenuName[21];
int		EntryNbr;

// global values for menu operations
uint8_t CurrentLevel;
uint8_t IndexCount;						// number of entries for current level
uint8_t IndexList[MAX_LEVEL_ENTRIES];	// indexes of current level entries
int8_t  CurrentIndexes[NBR_OF_LINES+1];	// menu index for each displayed entry
int		Timeout;						// updated by users interrupt
char	LCDstring[21];					// general string to print to the LCD
uint8_t	Invert_Dir = 0;					// 1 = invert DIR bit


// QE_Int is used by the QE clock ISR in pin_manager.c or ext_int.c
uint8_t QE_Int;


// navigate through the menu
// execute the selected menu item function
// copies the menu parameter to MenuParam
void DoMenu ( Menu_t * Menu )
{	uint16_t SwValue;
	int8_t	Line1List_x;	// index into IndexList of LCD line 1 menu entry
	uint8_t i, line;
	int8_t MenuIndex;
	int8_t (*FunctionPtr)(int);
	int MenuParam;
	
	line = LCD_Line();
	FillIndexList(Menu, 1);				// get indexes of all entries for this level
	Line1List_x = 0;					// index of IndexList of 1st LCD line
	ShowMenuEntries ( Menu, Line1List_x );
	LCD_Pos(1,1);
	while (1)
	{	LCD_Cursor (CURSOR_BLOCK);
		line = LCD_Line();
		SwValue = WaitSwitches();		// wait for a switch
		if ( !SwValue ) return;			// must be a timeout <<<<<<<<<<<<<<<<<<<
		if ( SwValue == CW )			// if CW rotation
		{	if ( IndexCount <= NBR_OF_LINES ) // if short list
			{	if ( line == IndexCount ) LCD_Pos(1,1);
				else LCD_Pos(line+1,1);
			}
			else if ( line < NBR_OF_LINES ) // if NOT on last LCD line
			{	LCD_Pos (line+1,1); // next line
			}
			else						// if LCD screen is full
			{	Line1List_x++;			// scroll top line off screen
				if ( Line1List_x >= IndexCount ) // if no more entries
					Line1List_x = 0;	// goto beginning of list
				ShowMenuEntries ( Menu, Line1List_x );
				LCD_Pos(4,1);
			}
		} // SwValue == CW
		if ( SwValue == CCW )			// if CW rotation
		{	if ( IndexCount <= NBR_OF_LINES ) // if short list
			{	if ( line == 1 ) LCD_Pos(IndexCount,1);
				else LCD_Pos(line-1,1);
			}
			else if ( line > 1 )		// if NOT on first LCD line
			{	LCD_Pos (line-1,1);		// previous line
			}
			else
			{	Line1List_x--;			// scroll bottom line off screen
				if ( Line1List_x < 0 )	// if no more entries
					Line1List_x = IndexCount-1; // goto end of list
				ShowMenuEntries ( Menu, Line1List_x );
				LCD_Pos(1,1);
			}
		} // SwValue == CCW
			
		if ( SwValue == SW_QE || SwValue == SW_2 ) // select this menu entry
		{	MenuIndex = CurrentIndexes[line-1]; // menu index of selected entry
			FunctionPtr = Menu[MenuIndex].Funct;
			if ( FunctionPtr == 0 )		// if this is a "parent" entry
			{	MenuIndex++;			// point to next menu entry
				i = Menu[MenuIndex].Level; // get level value
				if (!i) return;			// level 0 = exit menu <<<<<<<<<<<<<<<<< 
				FillIndexList(Menu, MenuIndex); // get indexes of all entries for new level
				Line1List_x = 0;		// start at top of IndexList
			}
// *****************************************************************************				
			else
			{
				LCD_Cursor ( CURSOR_DEFAULT );
				LCD_Clear();
				MenuParam = Menu[MenuIndex].Param;
				FunctionPtr(MenuParam);	// >>>>>>>> execute the function <<<<<<<
				Delay(100);
			}
// *****************************************************************************				
			ShowMenuEntries ( Menu, Line1List_x );
			LCD_Pos(1,1);
		}
		
		if ( SwValue == SW_3 )			// back up one level
		{	if ( CurrentLevel == 1 ) return; // <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
			i = IndexList[0];			// get menu index of 1st entry this level
			i--;						// backup one menu entry
			FillIndexList(Menu, i);		// get indexes of all entries for this level
			Line1List_x = 0;			// point to 1st entry in list
			ShowMenuEntries ( Menu, Line1List_x );
			LCD_Pos(1,1);
		}  // SwValue
	}	
} // DoMenu


// FillIndexList
// fill the index list for this menu level
// entry argument: index of the 1st menu entry this current level
// Operation:
//		skip entries with a higher level
//		terminate search when a lower level entry is found
// global values set:
//		IndexCount = nbr of entries for current level
//		IndexList = list of index entries for current level
//		CurrentLevel = level of menu entries being displayed
void FillIndexList ( Menu_t * Menu, uint8_t Index )
{	int8_t Level;

	memset ( IndexList, 0, MAX_LEVEL_ENTRIES ); // clear IndexList
	CurrentLevel = Menu[Index].Level;
// make sure to start with 1st entry for CurrentLevel
	while ( Menu[--Index].Level >= CurrentLevel );
	Index++;
// now find all entries
	IndexCount = 0;						// init nbr of entries
	while (1)
	{	Level = Menu[Index].Level;	// get this entry level
		if ( Level == CurrentLevel )	// if entry is same level
		{	IndexList[IndexCount] = Index;
			IndexCount++;				// update nbr of entries
		}
		if ( Level < CurrentLevel ) break; // exit if lower level entry
		// skip entry if level > CurrentLevel
		Index++;						// point to next entry
	}
} // FillIndexList


// show set of menu entries
// entry argument: index into IndexList of 1st entry to display
// uses globals: IndexCount
// fills CurrentIndexes array: menu index for each displayed entry
void ShowMenuEntries ( Menu_t * Menu, uint8_t Index )
{	int8_t MenuIndex, line;

	LCD_Clear();
	line = 0;							// init for loop
	while ( 1 )
	{	MenuIndex = IndexList[Index];	// menu index for this entry
		CurrentIndexes[line] = MenuIndex; // save it
		LCD_Pos ( ++line, 2 );			// place cursor on line for this entry
		LCD_Print (Menu[MenuIndex].Name);
		if ( line >= IndexCount ) break; // exit if all entries displayed
		if ( line >= NBR_OF_LINES ) break; // exit if on last LCD line
		Index++;						// point to next menu entry
		if ( Index >= IndexCount ) Index = 0; // start over if at end of list
	}
	CurrentIndexes[line+1] = -1;		// show end of list
} // ShowMenuEntries


#ifdef EN_ENTERTEXT
// get text from the operator
uint8_t EnterText (char *Text, const char* ValidChars, uint8_t MaxChars)
{	int VCI, VCImax, CharCount, SwValue;
	// VCI: Valid Character Index
	uint8_t line, column;

	LCD_Cursor (CURSOR_UNDERLINE);
	VCImax = strlen (ValidChars);
	line = LCD_Line();					// save current
	column = LCD_Column();				//		cursor position
	LCD_Pos ( 4,1 );
	LCD_Print ("Sw1=Accept  Sw2=Done");
	memset ( Text, 0, MaxChars );

	CharCount = VCI = 0;
	while (CharCount < MaxChars )
	{	LCD_Pos (line,column+CharCount); // character selection position
		LCD_Char (ValidChars[VCI]);		// print candidate character
		LCD_Pos (0,column+CharCount);	// backup to character position

		SwValue = WaitSwitches();		// wait for a switch
		if ( (SwValue == SW_QE) )		// accept the character
		{	Text[CharCount] = ValidChars[VCI]; // save it
			CharCount++;				// point to next character position
			VCI = 0;					// start at beginning of character list
		}
		else if (SwValue == CW)
		{	VCI++;						// index of next char in list
			if ( VCI >= VCImax ) VCI = 0; // if end of list: go to beginning
		}
		else if (SwValue == CCW)
		{	VCI--;						// index of previous char in list
			if ( VCI < 0 ) VCI = VCImax-1; // if beginning of list: goto end
		}
		else if ( SwValue == SW_2 )		// if operator is done
		{	Text[CharCount] = 0;		// null terminator
			break;			
		}
		else if ( SwValue == SW_3 )		// delete previous character
		{	if ( CharCount>0 )
			{	Text[--CharCount] = 0;	// remove from text and update count
				VCI = 0;
			}
			else break;
		}
	}
	LCD_Cursor (CURSOR_DEFAULT);
	return SwValue;
} // EnterText
#else
uint8_t EnterText (char *Text, const char* ValidChars, uint8_t MaxChars)
{	return 0; }
#endif // EN_ENTERTEXT


#ifdef EN_GETNUMBER
// get a number
// enter with the cursor at the desired position
// the number of digits specified must fit on the line
#define MAX_DIGITS	10
int8_t GetNumber ( long int* Value, uint8_t Digits, char* suffix, void (*FunctionPtr)() )
{	uint8_t SwValue, DigitPos, i, NegFlag;
	uint8_t column, row;
	long int DigitMult, MaxDigitMult, Number;
	char DigitString[MAX_DIGITS+1];

	Number = *Value;
	LCD_Cursor (CURSOR_UNDERLINE);
	column = LCD_Column();				// get current cursor position
	row = LCD_Line();
	DigitPos = column + Digits-1;		// units digit position
	DigitMult = 1;
	MaxDigitMult = 1;					// calculate maximum multiplier value
	for (i=2; i<=Digits; i++) MaxDigitMult*=10; // 10**Digits
	NegFlag = 0;						// init to no negative values yet
	if ( FunctionPtr ) (*FunctionPtr)(*Value); // execute callback if present
	while (1)
	{
#ifdef EN_INT_TO_ASCII
		Int_to_ASCII ( *Value, Digits, DigitString, 0, 0 );
#else
		sprintf (DigitString,"%*.*li", Digits, Digits, *Value);
#endif
		LCD_Pos ( row, column );
		if ( *Value < 0 )
		{	LCD_Char('-');
			if ( !NegFlag ) NegFlag=1, MaxDigitMult /= 10; // 1 less digit
		}
		LCD_Print (DigitString);
		LCD_Print ( suffix );
		LCD_Pos ( row, DigitPos );
		SwValue = WaitSwitches();
		if ( SwValue == SW_QE ) break;	// exit if done
		if ( SwValue == CW )			// if CW rotation
		{	*Value += DigitMult;
			if ( *Value >= MaxDigitMult*10 ) *Value = MaxDigitMult*10-1;
		}
		if ( SwValue == CCW )			// if CCW rotation
		{	*Value -= DigitMult;
			if ( *Value <= -MaxDigitMult*10 ) *Value = -MaxDigitMult*10+1;
		}
		if  ( SwValue == SW_3 )			// move to lesser significant digit
		{	if ( DigitMult > 1 )		// if valid
			{	DigitMult /= 10;
				DigitPos += 1;			
			}
			else return 0;
		}
		if  ( SwValue == SW_2 )			// move to more significant digit
		{	if ( DigitMult < MaxDigitMult ) // if valid
			{	DigitMult *= 10;
				DigitPos -= 1;			
			}
		}
		if ( FunctionPtr ) (*FunctionPtr)(*Value); // execute callback if present
	}
	LCD_Pos (row,column+Digits);		// cursor to 1 position past digits
	LCD_Cursor (CURSOR_DEFAULT);
	return 1;							// show success
} // GetNumber
#else
int8_t GetNumber ( long int* Value, uint8_t Digits, char* suffix, void (*FunctionPtr)() )
{	return 0; }
#endif // EN_GETNUMBER


#ifdef EN_SELECT_N_ON_LINE
// select n of m options from a single line of text
// enter with:
//		initial option value
//		array of column nbrs of each option, terminated with 0
//			maximum nbr of items = 16
// IMPORTANT: bit 0 is for the left-most option
// LCD screen must be set as desired
uint16_t Select_N_on_Line ( uint16_t InitialValue, const uint8_t Columns[] )
{	int	bitnbr;
	uint16_t SwValue, NewValue;
	uint8_t size;

	LCD_Cursor (CURSOR_UNDERLINE);
	for ( size=0; size<16; size++ ) if (Columns[size] == 0) break;
										// determine nbr of items on line
	NewValue = InitialValue;
	bitnbr = 0;
	do
	{	LCD_Pos(0,Columns[bitnbr]);		// place cursor under option
		if ( InitialValue & (1<<bitnbr) ) LCD_Char('Y');
		else LCD_Char('N');
		bitnbr++;
	} while (Columns[bitnbr]);

	bitnbr = 0;
	while (1)
	{	LCD_Pos(0,Columns[bitnbr]);
		SwValue = WaitSwitches();		// wait for a switch
		if ( SwValue == SW_2 ) break;	 // exit if done
		if ( SwValue == CW )
		{	bitnbr++;					// next bit nbr
			if ( bitnbr >= size ) bitnbr = 0;			
		}
		if ( SwValue == CCW )
		{	bitnbr--;					// previous bit nbr
			if ( bitnbr < 0 ) bitnbr = size-1;
		}
		if ( SwValue == SW_QE )			// if toggle this bit
		{	NewValue ^= 1<<bitnbr;		// invert the bit			
			if ( NewValue & (1<<bitnbr) ) LCD_Char('Y');
			else LCD_Char('N');
			bitnbr++;					// point to next bit nbr
			if ( bitnbr >= size ) bitnbr = 0;			
		}
	}
	LCD_Cursor (CURSOR_DEFAULT);
	return NewValue;
} // Select_N_on_Line
#else
uint16_t Select_N_on_Line ( uint16_t InitialValue, const uint8_t Columns[] )
{	return 0; }
#endif // EN_SELECT_N_ON_LINE


#ifdef EN_SELECT_1_ON_LINE
// Enable the operator to select from among the displayed options
// The options must be on a single line
// entry arguments:
//		array of column nbrs of each option, terminated with 0
//			maximum nbr of items = 8
// return value:
//		normal: the index within the array of the selection
//		SW3 exit -1;
// the text must already be displayed
int8_t	Select_1_on_Line ( const uint8_t Columns[] )
{	uint8_t SwValue;
	int8_t Index, i, size;

	LCD_Cursor (CURSOR_UNDERLINE);
	for ( size=0; size<=8; size++ ) if (Columns[size] == 0) break;
										// determine nbr of items on line
	size -= 1;							// size = index of last non-0 entry
	Index = 0;
	do
	{	i = Columns[Index];
		LCD_Pos ( 0, i );
		SwValue = WaitSwitches();		// wait for a switch
		if ( SwValue == CW  ) Index++;
		if ( SwValue == CCW ) Index--;	// Index = 0xFF if underflow
		if ( Index > size  ) Index = 0;
		if ( Index < 0 ) Index = size;
		if ( SwValue == SW_3 )
		{	Index = -1;
			break;
		}
	} while ( (SwValue&(SW_QE) ) == 0 );
	LCD_Cursor (CURSOR_DEFAULT);
	return Index;
} // Select_1_on_Line
#else
int8_t	Select_1_on_Line ( const uint8_t Columns[] )
{	return 0; }
#endif // EN_SELECT_1_ON_LINE


#ifdef EN_SELECT_1_OF_LIST
// Enable the operator to select an entry from a list
// entry arguments:
//		a char* array containing the options
//		the number of options
//		address of title, 0=no title
//	The caller should have title text on line 1 of the LCD and the remaining
//		lines cleared.
// return value:
//		normal: the index within the array of the selected option
//		SW3 exit: -1
int8_t	Select_1_of_List ( char *OptionList[], uint8_t size, char *Title )
{	uint8_t SwValue, TopLine, i, renew;
	uint8_t CursorLine, ShortList;
	uint8_t IndexList[5];
			// contains the OptionList index of the item
			// displayed on the corresponding LCD line. [0] is not used]
	uint8_t Index;

	LCD_Cursor(CURSOR_BLOCK);
	Index = 0;							// index into OptionList
	if ( Title ) TopLine = 2;
	else TopLine = 1;
	ShortList =  ( size <= (NBR_OF_LINES-TopLine+1) );
						// ShortList = 1 if all options fit on screen at once
// store the index numbers for the 1st set of list entries to be displayed
	for ( Index = TopLine; Index <= NBR_OF_LINES; Index ++ )
		IndexList[Index] = Index-TopLine;

	CursorLine = TopLine;
	renew = 1;							// force screen update
	do
	{	if ( renew )
		{	LCD_Clear ();
			if ( Title ) LCD_Print ( Title ); // if there is a title
			LCD_LineNbr = TopLine;
		// display up to NBR_OF_LINES entries
			for ( LCD_LineNbr = TopLine; LCD_LineNbr <= NBR_OF_LINES; LCD_LineNbr++ )
			{	LCD_Pos ( 0, 2 );
				i = IndexList[LCD_LineNbr];	// get index into list
				if ( i >= size ) break;
				LCD_Print ( OptionList[i] );
			}
		}
		LCD_Pos ( CursorLine, 1 );
		renew = 0;
		SwValue = WaitSwitches();		// wait for a switch
		if ( SwValue == CW )
		{	if ( ShortList  &&  (IndexList[LCD_LineNbr] == size-1) )
				CursorLine = TopLine-1;
			if ( CursorLine < NBR_OF_LINES ) CursorLine++;
			else
			{	for ( Index = TopLine; Index <= NBR_OF_LINES; Index ++ )
				{	IndexList[Index]++;
					if ( IndexList[Index] >= size ) IndexList[Index] = 0;
				}
				renew = 1;				// force screen update
			}
		}
		if ( SwValue == CCW )
		{	if ( ShortList  &&  (LCD_LineNbr == TopLine) )
				CursorLine = TopLine+size;
			if ( CursorLine > TopLine ) CursorLine--;
			else
			{	for ( Index = TopLine; Index <= NBR_OF_LINES; Index ++ )
				{	IndexList[Index]--;
					if ( IndexList[Index] == 0xFF ) IndexList[Index] = size-1;
				}
				renew = 1;				// force screen update
			}
		}
		if ( SwValue == SW_3 return -1;
	} while ( SwValue != SW_QE );
	LCD_Cursor ( CURSOR_DEFAULT );
	return IndexList[LCD_LineNbr];
} // Select_1_of_List
#else
int8_t	Select_1_of_List ( char *OptionList[], uint8_t size, char *Title )
{	return 0; }
#endif // EN_SELECT_1_OF_LIST


#ifdef EN_YES_NO
// Get a Yes or No from the operator
// Return value: 1 if Yes,  0 if no
uint8_t Indexes_YN[] = {1,5,0};
uint8_t Yes_No ( void )
{	uint8_t y_n, column;

	column = LCD_Column();
	Indexes_YN[0] = column+1;
	Indexes_YN[1] = column+5;
	LCD_Print ("Yes/No");
	y_n = Select_1_on_Line ( Indexes_YN );
	y_n = 1 - y_n;
	return y_n;
} // Yes_No
#else
uint8_t Indexes_YN[] = {1,5,0};
{	return 1; }
#endif // EN_YES_NO


#ifdef EN_INT_TO_ASCII
uint8_t Int_to_ASCII ( long int number, int8_t size, char* text, char* prefix, char* suffix )
{	int8_t i,j, negflag;
	char* pointer;

	pointer = text;						// save starting address
	i = 0;								// index into text
	if ( prefix )						// if there is a prefix
		while ( *prefix ) text[i++] = *prefix++; // copy the prefix to the output
	if ( size > 0 ) memset ( &text[i], ' ', size ); // fill with spaces
	else
	{	size = -size;					// make positive
		 memset ( &text[i], '0', size );	// fill with 0s
	}
	i += size;							// point to 1 char past digits
	j = i;								// save it
	negflag = 0;						// assume positive number
	if ( number < 0 ) number = -number, negflag = 1;
	while ( number )
	{	text[--j] = (number%10) + '0';
		number /= 10;
	}
	if ( negflag && (&text[j] > pointer) ) text[--j] = '-';
	if ( suffix )						// if there is a prefix
		while ( *suffix ) text[i++] = *suffix++; // copy the suffix to the output
	text[i] = 0;						// insert trailing null
	return i;
} // Int_to_ASCII
#else
	uint8_t Int_to_ASCII ( long int number, int8_t size, char* text, char* prefix, char* suffix )
{	return 0; }
#endif // EN_INT_TO_ASCII


#ifdef EN_FP_TO_ASCII
uint8_t FP_to_ASCII ( char* text, uint8_t size, uint8_t fract_digits, double fpvalue )
{	char *pntr;
	int32_t ivalue;
	int8_t i;

	pntr = text + size;					// point to
	*pntr-- = 0;						//		LS fraction digit
	memset ( text, ' ', size );			// fill with spaces
	if ( fpvalue < 0.0 )				// if value is negative
	{	text[0] = '-';					// insert '-' sign
		fpvalue = -fpvalue;				// make positive
		size -= 1;						// one less position for digits
	}
	for ( i=0; i<fract_digits; i++ ) fpvalue *= 10.0;
	ivalue = fpvalue;
	for ( i=1; i<size; i++ )			// must exclude '.'
	{	*pntr-- = ivalue%10 | 0x30;		// get a digit
		ivalue /= 10;
		if ( --fract_digits == 0 ) *pntr-- = '.';
	}
	return 0;
} // FP_to_ASCII
#endif


uint8_t flag = 0;
// wait for ANY switch - with debounce
uint8_t WaitSwitches ( void )
{	uint16_t SwValue;

	QE_Int = 0;							// enable QE interrupt
	SwValue = 0;
	Timeout = 60;						// allow up to 1 minute
	while ( !SwValue & ( Timeout>0) )
	{	if ( QE_Int )
		{	Delay(2);					// some debounce time
			if (SW_QE_CLK)
			{	SwValue = QE_CLK | (QE_DIR * SW_QE_DIR);
				if ( Invert_Dir ) SwValue ^= QE_DIR;
			}
			else QE_Int = 0;
		}
		else
		{	flag = 1;					// enable Delay to exit if QE_Int
			SwValue = ReadPBs();		// get the value
			if (SwValue)
				while (ReadPBs());		// wait until release
			flag = 0;					// disable Delay exit if QE_Int
		}
	}
	return (uint8_t)SwValue;
} // WaitSwitches


// Wait for any combination of switches: press and release:
// SW_QE/SW_1, SW_2 and/or SW_3
uint8_t WaitPB ( uint8_t sw )
{	uint8_t i;
	while ( !(i = (ReadPBs() & sw) ) );	// wait for sw closure
	while ( (ReadPBs() & sw) );			// wait for sw release
	return i;
} // WaitSw


// ReadPBs
// read the Push Button switches - with debounce
// IMPORTANT: this function does NOT wait - it reads the switches and returns
// NOTE: the switches are assumed to be low true with pull-up resistors,
//		therefore, this function inverts the state so that 1=True
uint8_t ReadPBs (void)
{	uint8_t SwValue, i;
// read the switches and invert their state
	i  = (!SW1_BIT) * SW_1;
	i |= (!SW2_BIT) * SW_2;
	i |= (!SW3_BIT)	* SW_3;
	Delay (10);							// debounce time
// read the switches again
	SwValue  = (!SW1_BIT) * SW_1;
	SwValue |= (!SW2_BIT) * SW_2;
	SwValue |= (!SW3_BIT) * SW_3;
// ensure same value both times
	if (i != SwValue) SwValue = 0;		// show no switch if values are different
	return SwValue;	
} // ReadPBs


#define ONE_MS_DIVISOR 20089			// determined experimentally
#define ONE_MS	(_XTAL_FREQ/ONE_MS_DIVISOR) // count for 1ms delay
// delay some number of milliseconds
void Delay (uint16_t x)
{	uint16_t Count;
	do
	{	for ( Count=0; Count<ONE_MS; Count++ ) // 1ms
			if ( flag && QE_Int ) return;
	} while ( --x );
}
