/* 
File:   Project.h
Author: Larry Cicchinelli

Created on November 8, 2018, 7:21 PM
 
EEPROM data
The EEPROM starts at 0x7FFE00, page number = 0x7F, beginning offset = 0xFE00
IMPORTANT: All EEPROM operations are 16 bit words.

The EE_... macros define the complete address for the parameter.
The b... and w... macros define the byte and word offset into the EEPROM array
 
System Data structure:
	struct
	{ // IMPORTANT: Tmr3_Divlo MUST be first: see Read_Alarm and Write_Alarm
		// put words on even address
		uint16_t	Tmr3_Divlo;		// Timer divisor low order word
		uint16_t	Tmr3_Divhi;		// Timer divisor high order word
		uint16_t	Bright_tm_day;	// daytime brightness time of day
		uint16_t	Bright_tm_nite;	// dnitetime brightness time of day
		struct
		{	uint16_t day	: 5;	// day of monht
			uint16_t month	: 4;	// month of year
			uint16_t year	: 6;	// year offset from 2019
		}Date;
	// put bytes starting here
		// put bytes starting here
		uint8_t Token;
		struct
		{	uint8_t Hr24	: 1;	// 1 = 24 Hour format
			uint8_t Cal		: 1;	// 1 = calibrated
			uint8_t DST_En	: 1;	// 1 = DST enabled
			uint8_t DST		: 1;	// 1 = currently in DST
			uint8_t Secs	: 1;	// 1 = display seconds
			uint8_t Format	: 1;	// 1 = D/M/Y, 0 = M/D/Y	DATE FORMAT
		}Flag;
		uint8_t		Snooze;			// SNOOZE TIME IN MINUTES
		uint8_t		Bright_Day;		// daytime brightness level
		uint8_t		Bright_Nite;	// nighttime brightness level
		uint8_t		EZ_Days;		// EZ alarm: days of week for 
		uint8_t		EZ_Tone;		// EZ alarm: tone frequency
		uint8_t		EZ_Stop;		// EZ alarm: stop offset in minutes
		uint8_t		EZ_Repeat;		// EZ alarm: repeat weekly flag
		uint8_t		TMR_Tone;		// EZ timer: tone frequency
		uint8_t		TMR_Stop;		// EZ timer: stop offset in minutes
	} Sys;

Alarm/Timer data structure:
	struct
	{ // IMPORTANT: Start MUST be first: see Read_Alarm and Write_Alarm
		// put words on even address

		uint16_t Start;				// start time of the Alarm/timer
		uint16_t Stop;				// stop time
		struct
		{	uint8_t Repeat	: 1;	// 1 = Alarm/Timer repeats weekly
			uint8_t EZ_Timer : 1;	// 1 = this is an Easy Timer
			uint8_t On		: 1;	// 1 = Alarm/Timer is On
			uint8_t Cycle	: 1;	// 1 = Timer has on-off cycles
			uint8_t Enabled	: 1;	// 1 = Alarm/timer is enabled
			uint8_t Setup	: 1;	// 1 = Alarm/timer has been setup
		}Mode;

		union		// union is needed in order to be able to access the entire byte
		{	struct
			{	uint8_t Spkr	: 1;	// 1 = speaker is enabled
				uint8_t O1		: 1;	// 1 = output 1 is enabled
				uint8_t O2		: 1;	// 1 = output 2 is enabled
				uint8_t O3		: 1;	// 1 = output 4 is enabled
				uint8_t O4		: 1;	// 1 = output 4 is enabled
			}f;			// individual bits
			uint8_t b;	// entire byte
		} Outputs;
		uint8_t Days;				// days of the week that Alarm/Timer is active
		uint8_t Tone;				// Alarm tone frequency/8
		uint8_t Cycle_On;			// Timer cycle on time
		uint8_t Cycle_Off;			// Timer cycle off time
	}Alarms[NBR_OF_ALARMS];
*/

#ifndef PROJECT_H
#define	PROJECT_H

//#define DEBUG


// These #includes are required and must not be changed.  Most of the source
//	files have a #include Project.h.
#include "mcc_generated_files/mcc.h"
#include "LCD.h"
#include "Menu_IO.h"					// must precede Menu.h
#include "Menu.h"
#include <string.h>
#include <libpic30.h>					// for delay & EEPROM functions

#define MAX_POWER_CYCLES 10

// these values may be customized as required
// Snooze time defaults
#define DEF_SNOOZE_TIME	10				// default snooze time
#define MAX_SNOOZE		60				// maximum allowed snooze time

// CYCLE_RESOLUTION is the  resolution used when you enable the cycling feature
// of a timer.  It affects the On and OFF times since only 1 byte is used to
// store the on and off times.  The maximum value of the on and off times is:
//		CYCLE_RESOLUTION * 255
#define CYCLE_RESOLUTION	1			// max value = 6
#define DEFAULT_CYCLE_TIME	10			// 10 minutes

// Easy Alarm setup defaults
#define EASY_DAYS			0x3E		// 0011 1110: Mon-Fri, Sun = bit 0
#define EASY_ALARM_TONE		440			// is /8 when stored
#define EASY_ALARM_STOP		1			// nbr of minutes the alarm will ring
#define EASY_REPEAT			1			// 1=repeat weekly, 0=do not repeat

// Easy Timer setup defaults
#define EASY_TIMER_TONE		520			// is /8 when stored
#define EASY_TIMER_STOP		1			// nbr of minutes the alarm will ring

// LCD brightness setup defaults
#define BRIGHT_1_PWM		50
#define BRIGHT_2_PWM		10
#define BRIGHT_1_AD			200
#define BRIGHT_2_AD			10

// define the default cursor type for your functions
#define CURSOR_DEFAULT CURSOR_UNDERLINE	// must be one of:
								// CURSOR_UNDERLINE,  CURSOR_BLOCK,  CURSOR_OFF

// the following value must NOT be changed
#define OC3pin	_LATB0

#endif	/* PROJECT_H */

