/* Microchip Technology Inc. and its subsidiaries.  You may use this software 
 * and any derivatives exclusively with Microchip products. 
 * 
 * THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS".  NO WARRANTIES, WHETHER 
 * EXPRESS, IMPLIED OR STATUTORY, APPLY TO THIS SOFTWARE, INCLUDING ANY IMPLIED 
 * WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS FOR A 
 * PARTICULAR PURPOSE, OR ITS INTERACTION WITH MICROCHIP PRODUCTS, COMBINATION 
 * WITH ANY OTHER PRODUCTS, OR USE IN ANY APPLICATION. 
 *
 * IN NO EVENT WILL MICROCHIP BE LIABLE FOR ANY INDIRECT, SPECIAL, PUNITIVE, 
 * INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE, COST OR EXPENSE OF ANY KIND 
 * WHATSOEVER RELATED TO THE SOFTWARE, HOWEVER CAUSED, EVEN IF MICROCHIP HAS 
 * BEEN ADVISED OF THE POSSIBILITY OR THE DAMAGES ARE FORESEEABLE.  TO THE 
 * FULLEST EXTENT ALLOWED BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS 
 * IN ANY WAY RELATED TO THIS SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF 
 * ANY, THAT YOU HAVE PAID DIRECTLY TO MICROCHIP FOR THIS SOFTWARE.
 *
 * MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE OF THESE 
 * TERMS. 
 */

/* 
 * File:
 * Author: 
 * Comments:
 * Revision history: 
 */

// This is a guard condition so that contents of this file are not included
// more than once.  
#ifndef LCD_H
#define	LCD_H

#define	NBR_OF_LINES 4
#define NBR_OF_COLS 20

void LCD_Initialize (void);			// in LCD_IO.c, called from main

// functions within LCD.C
// Note: Those Characters which would display beyond end of the NBR_OF_COLS character
//		line will be discarded.

// Send a character to the LCD
// will not print characters beyond NBR_OF_COLS
// special cases:
//		\n = goto to next line column 1
//		\r = goto column 1 of current line
//		\1..\4 = goto line x, column 1
// entry argument: the character to be printed
// exit with: LCD_ColNbr incremented if valid
void LCD_Char (char);

// Clears the LCD
// Sets LCD_LineNbr = 1 and LCD_ColNbr = 1
void LCD_Clear (void);

// clear the specified line number
// entry argument: Line nbr to be cleared
//		0 = current line
// exit with: the cursor in column 1
void LCD_ClearLine (uint8_t);

// set cursor type
// entry argument: UNDERLINE or BLOCK
// note: any function that sets the cursor to BLOCK must set it to UNDERLINE
//	before exiting.
void LCD_Cursor (uint8_t);

// set the LCD cursor position
// entry arguments:
//		Line number, 1 relative: 1..NBR_OF_LINES
//			0 = current line
//		Column number, 1 relative: 1..NBR_OF_COLS
void LCD_Pos (uint8_t, uint8_t);

// get the current cursor position
uint8_t LCD_Line (void);
uint8_t LCD_Column (void);

// print the specified text
// entry argument: address of text to print - must be NULL terminated
// will not print characters beyond NBR_OF_COLS
// special cases: see LCD_Char
// updates LCD_ColNbr
void LCD_Print (char*);

// print the specified text
// entry arguments:
//		address of text to print
//		number of characters to print
// will not print characters beyond NBR_OF_COLS
// special cases: see LCD_Char
// updates LCD_ColNbr
void LCD_PrintN	(char*, uint8_t);


#define CURSOR_UNDERLINE	2
#define CURSOR_BLOCK		1
#define CURSOR_OFF			0

#endif	/* LCD_H */
